/*
 * This file is part of Remote X11 (RX11), a library and server that lets
 * you develop applications on systems without X11 support.
 *
 * Copyright (C) 1995, Kay Roemer. See COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "RXlib.h"


RXDisplay *dpy;
int screen;
Window w;
RXGC gc;

u_long blue_pixel, yellow_pixel;

void
AllocColors (void)
{
  RXColor s, e;
  Colormap cmap = RXDefaultColormap (dpy, screen);

  if (!RXAllocNamedColor (dpy, cmap, "blue", &s, &e))
    printf ("cannot alloc blue\n");
  else
    blue_pixel = e.pixel;

  if (!RXAllocNamedColor (dpy, cmap, "yellow", &s, &e))
    printf ("cannot alloc yellow\n");
  else
    yellow_pixel = e.pixel;
}

void
Xlogo (void)
{
  u_int wd, ht;
  int xhot, yhot;
  char *data, *data2;
  Pixmap pix;

  AllocColors ();

  data2 = RXReadXbmFile (dpy, "xlogo32.xbm", &wd, &ht, &xhot, &yhot);
  if (!data2) {
    printf ("cannot read xlogo32.xbm\n");
    return;
  }
  data = RXScaleBitmapData (dpy, data2, &wd, &ht, 2, 2);
  if (!data) {
    printf ("cannot scale bitmap\n");
    return;
  }
  pix = RXCreatePixmapFromBitmapData (dpy, w, data, wd, ht,
				      blue_pixel, yellow_pixel,
				      RXDefaultDepth (dpy, screen));
  RXCopyArea (dpy, pix, w, gc, 0, 0, wd, ht,
	      wd > 100 ? 0 : (100-wd)/2,
	      ht > 100 ? 0 : (100-ht)/2);
  RXFree (data);
  RXFree (data2);
}

void
Text (void)
{
  RXDrawString (dpy, w, gc, 5, 20, "Hello, world!", strlen ("Hello, world!"));
}

void
Points (void)
{
  static int pts[100*100];
  int i;

  for (i = 0; i < 100*100; ++i)
    pts[i] = i;

  for (i = 0; i < 100*100; ++i) {
    int j = rand () % (100*100);
    int t = pts[i];
    pts[i] = pts[j];
    pts[j] = t;
  }
  for (i = 0; i < 100*100; ++i)
    RXDrawPoint (dpy, w, gc, pts[i] % 100, pts[i] / 100);
}

void
Lines (void)
{
  RXGCValues val;
  int i;

  val.function = GXxor;
  val.foreground = RXBlackPixel (dpy, screen) ^ RXWhitePixel (dpy, screen);
  RXChangeGC (dpy, gc, GCFunction|GCForeground, &val);

  for (i = 0; i < 10000; ++i) {
    int x1 = rand () % 100;
    int y1 = rand () % 100;
    int x2 = rand () % 100;
    int y2 = rand () % 100;
    RXDrawLine (dpy, w, gc, x1, y1, x2, y2);
  }
}

int
main (int argc, char *argv[])
{
    RXGCValues val;

    dpy = RXOpenDisplay (NULL);

    if (argc != 2)
      exit (0);
    
    if (!dpy) {
      printf ("OpenDisplay failed\n");
      return 0;
    }
    screen = RXDefaultScreen (dpy);
    w = RXCreateSimpleWindow (dpy, RXRootWindow (dpy, screen),
			      20, 20, 100, 100,
			      2,
			      RXBlackPixel (dpy, screen),
			      RXWhitePixel (dpy, screen));
    RXStoreName (dpy, w, "RX11 test client");

    val.foreground = RXBlackPixel (dpy, screen);
    gc = RXCreateGC (dpy, w, GCForeground, &val);

    if (!strcmp (argv[1], "points"))
      Points ();
    else if (!strcmp (argv[1], "lines"))
      Lines ();
    else if (!strcmp (argv[1], "xlogo"))
      Xlogo ();
    else if (!strcmp (argv[1], "text"))
      Text ();

    while (fgetc (stdin) != EOF);

    RXDestroyWindow (dpy, w);
    RXFreeGC (dpy, gc);
    RXCloseDisplay (dpy);
    return 0;
}


