/*
 * This file is part of Remote X11 (RX11), a library and server that lets
 * you develop applications on systems without X11 support.
 *
 * Copyright (C) 1995, Kay Roemer. See COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <netinet/in.h>
#include "rx11.h"
#include "util.h"

#include "byteorder.h"

#define offsetof(str,el) ((long)&((str *)0)->el)

Request *
RequestAlloc (Request *r, Opcode op)
{
    if (!r)
      r = xmalloc (sizeof (Request));

    r->opcode = op;
    r->length = 0;
    r->ptr = r->req;

    return r;
}

void
RequestFree (Request *r)
{
    free (r);
}

int
RequestSend (int fd, Request *r)
{
    char *buf;
    long l;
    int ret;

    buf = (char *)r;
    l = offsetof (Request, req) + r->length;

    r->opcode = htonl (r->opcode);
    r->length = htonl (r->length);

    while (l > 0) {
	ret = write (fd, buf,
#ifdef __HELIOS__
		     l > 500 ? 500 : l);
#else
		     l);
#endif
	if (ret < 0) {
	    if (errno == EINTR)
	      continue;
	    perror ("RX11: write");
	    return -1;
	}
	l -= ret;
	buf += ret;
    }
    return 0;
}

int
RequestRecv (int fd, Request *r)
{
    int ret;
    int len;
    char *buf;

    len = offsetof (Request, req);
    buf = (char *)r;

    while (len > 0) {
	ret = read (fd, buf, len);
	if (ret < 0) {
	    if (errno == EINTR)
	      continue;
	    perror ("RX11: read");
	    return -1;
	} else if (ret == 0)
	  return 1; /* EOF */
	len -= ret;
	buf += ret;
    }
    r->opcode = ntohl (r->opcode);
    r->length = ntohl (r->length);
    r->ptr = r->req;

    len = r->length;
    while (len > 0) {
	ret = read (fd, buf, len);
	if (ret < 0) {
	    if (errno == EINTR)
	      continue;
	    perror ("RX11: read");
	    return -1;
	} else if (ret == 0)
	  return 1; /* EOF */
	buf += ret;
	len -= ret;
    }
    return 0;
}
