/*
 * This file is part of Remote X11 (RX11), a library and server that lets
 * you develop applications on systems without X11 support.
 *
 * Copyright (C) 1995, Kay Roemer. See COPYING for details.
 */

#ifndef _RX11_H
#define _RX11_H

#include <sys/types.h>
#include "RXlib.h"

#define MAX_REQ_SIZE 156000
#define RX11_PORT    12000

typedef enum {
    OpenDisplayReq,
    OpenDisplayRep,

    CloseDisplayReq,

    FlushReq,

    CreateSimpleWindowReq,
    CreateSimpleWindowRep,
    DestroyWindowReq,
    StoreNameReq,

    CreateGCReq,
    CreateGCRep,
    FreeGCReq,
    ChangeGCReq,

    CreatePixmapReq,
    CreateBitmapFromDataReq,
    CreatePixmapFromBitmapDataReq,
    CreatePixmapRep,
    FreePixmapReq,

    CopyAreaReq,
    CopyPlaneReq,
    CopyBitmapReq,

    DrawPointReq,
    DrawLineReq,
    DrawRectangleReq,
    DrawArcReq,
    FillRectangleReq,
    FillArcReq,
    DrawStringReq,
    DrawImageStringReq,

    AllocColorReq,
    AllocColorRep,
    AllocNamedColorReq,
    AllocNamedColorRep,
    QueryColorReq,
    QueryColorRep,

    ErrorRep,

    UpdateModeReq,
    FlushWindowReq
} Opcode;

typedef struct {
    long opcode;
    long length;
    char req[MAX_REQ_SIZE];
    char *ptr;
} Request;

extern int EncodeLong   (Request *, long);
extern int EncodeShort  (Request *, short);
extern int EncodeString (Request *, char *);
extern int EncodeOpaque (Request *, void *, long);

extern int EncodeRXVisual              (Request *, RXVisual *);
extern int EncodeRXScreen              (Request *, RXScreen *);
extern int EncodeRXDisplay             (Request *, RXDisplay *);
extern int EncodeRXGCValues            (Request *, RXGCValues *);
extern int EncodeRXSetWindowAttributes (Request *, RXSetWindowAttributes *);
extern int EncodeRXWindowAttributes    (Request *, RXWindowAttributes *);
extern int EncodeRXWindowChanges       (Request *, RXWindowChanges *);
extern int EncodeRXColor               (Request *, RXColor *);
extern int EncodeRXSegment             (Request *, RXSegment *);
extern int EncodeRXPoint               (Request *, RXPoint *);
extern int EncodeRXRectangle           (Request *, RXRectangle *);
extern int EncodeRXArc                 (Request *, RXArc *);

extern int DecodeLong   (Request *, long *);
extern int DecodeShort  (Request *, short *);
extern int DecodeString (Request *, char **);
extern int DecodeOpaque (Request *, void **, long *);

extern int DecodeRXVisual              (Request *, RXVisual *);
extern int DecodeRXScreen              (Request *, RXScreen *);
extern int DecodeRXDisplay             (Request *, RXDisplay *);
extern int DecodeRXGCValues            (Request *, RXGCValues *);
extern int DecodeRXSetWindowAttributes (Request *, RXSetWindowAttributes *);
extern int DecodeRXWindowAttributes    (Request *, RXWindowAttributes *,
					VisualID *);
extern int DecodeRXWindowChanges       (Request *, RXWindowChanges *);
extern int DecodeRXColor               (Request *, RXColor *);
extern int DecodeRXSegment             (Request *, RXSegment *);
extern int DecodeRXPoint               (Request *, RXPoint *);
extern int DecodeRXRectangle           (Request *, RXRectangle *);
extern int DecodeRXArc                 (Request *, RXArc *);

extern Request *RequestAlloc (Request *, Opcode);
extern void     RequestFree (Request *);

extern int      RequestSend (int, Request *);
extern int      RequestRecv (int, Request *);

#endif
