/*
 * This file is part of Remote X11 (RX11), a library and server that lets
 * you develop applications on systems without X11 support.
 *
 * Copyright (C) 1995, Kay Roemer. See COPYING for details.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "RXlib.h"
#include "rx11.h"
#include "funcs.h"

#define MAX_GC  16

static struct GCTabEntry {
  GC gc;
  int used;
} GCTab[MAX_GC];

RXGC
GCAdd (GC gc)
{
  int i;

  for (i = 0; i < MAX_GC; ++i) {
    if (!GCTab[i].used) {
      GCTab[i].used = 1;
      GCTab[i].gc   = gc;
      return i;
    }
  }
  fprintf (stderr, "GC table overflow\n");
  return 0;
}

void
GCDel (RXGC rgc)
{
  if ((u_long)rgc >= MAX_GC || !GCTab[rgc].used)
    fprintf (stderr, "GC index out of range\n");
  else
    GCTab[rgc].used = 0;
}

GC
GCLook (RXGC rgc)
{
  if ((u_long)rgc >= MAX_GC || !GCTab[rgc].used) {
    fprintf (stderr, "GC index out of range\n");
    return GCTab[0].gc;
  } else
    return GCTab[rgc].gc;
}

